@@ -3,7 +3,7 @@
 /*
 htop - AvailableColumnsPanel.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -14,7 +14,7 @@ typedef struct AvailableColumnsPanel_ {
    Panel* columns;
 } AvailableColumnsPanel;
 
-extern PanelClass AvailableColumnsPanel_class;
+extern const PanelClass AvailableColumnsPanel_class;
 
 AvailableColumnsPanel* AvailableColumnsPanel_new(Panel* columns);
 
