@@ -3,13 +3,13 @@
 /*
 htop - ColorsPanel.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "Panel.h"
-#include "Settings.h"
 #include "ScreenManager.h"
+#include "Settings.h"
 
 typedef struct ColorsPanel_ {
    Panel super;
@@ -18,7 +18,7 @@ typedef struct ColorsPanel_ {
    ScreenManager* scr;
 } ColorsPanel;
 
-extern PanelClass ColorsPanel_class;
+extern const PanelClass ColorsPanel_class;
 
 ColorsPanel* ColorsPanel_new(Settings* settings, ScreenManager* scr);
 
