@@ -3,7 +3,7 @@
 /*
 htop - FunctionBar.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -12,7 +12,10 @@ in the source distribution for its full text.
 typedef struct FunctionBar_ {
    int size;
    char** functions;
-   char** keys;
+   union {
+      char** keys;
+      const char* const* constKeys;
+   } keys;
    int* events;
    bool staticData;
 } FunctionBar;
@@ -25,9 +28,11 @@ void FunctionBar_delete(FunctionBar* this);
 
 void FunctionBar_setLabel(FunctionBar* this, int event, const char* text);
 
-void FunctionBar_draw(const FunctionBar* this, char* buffer);
+void FunctionBar_draw(const FunctionBar* this);
 
-void FunctionBar_drawAttr(const FunctionBar* this, char* buffer, int attr);
+void FunctionBar_drawExtra(const FunctionBar* this, const char* buffer, int attr, bool setCursor);
+
+void FunctionBar_append(const char* buffer, int attr);
 
 int FunctionBar_synthesizeEvent(const FunctionBar* this, int pos);
 
