@@ -3,13 +3,16 @@
 /*
 htop - IncSet.h
 (C) 2005-2012 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include <stdbool.h>
+#include <stddef.h>
+
 #include "FunctionBar.h"
 #include "Panel.h"
-#include <stdbool.h>
+#include "Vector.h"
 
 #define INCMODE_MAX 40
 
@@ -18,10 +21,8 @@ typedef enum {
    INC_FILTER = 1
 } IncType;
 
-#define IncSet_filter(inc_) (inc_->filtering ? inc_->modes[INC_FILTER].buffer : NULL)
-
 typedef struct IncMode_ {
-   char buffer[INCMODE_MAX+1];
+   char buffer[INCMODE_MAX + 1];
    int index;
    FunctionBar* bar;
    bool isFilter;
@@ -35,6 +36,10 @@ typedef struct IncSet_ {
    bool found;
 } IncSet;
 
+static inline const char* IncSet_filter(const IncSet* this) {
+   return this->filtering ? this->modes[INC_FILTER].buffer : NULL;
+}
+
 typedef const char* (*IncMode_GetPanelValue)(Panel*, int);
 
 void IncSet_reset(IncSet* this, IncType type);
@@ -43,17 +48,13 @@ IncSet* IncSet_new(FunctionBar* bar);
 
 void IncSet_delete(IncSet* this);
 
-bool IncSet_next(IncSet* this, IncType type, Panel* panel, IncMode_GetPanelValue getPanelValue);
-
-bool IncSet_prev(IncSet* this, IncType type, Panel* panel, IncMode_GetPanelValue getPanelValue);
-
 bool IncSet_handleKey(IncSet* this, int ch, Panel* panel, IncMode_GetPanelValue getPanelValue, Vector* lines);
 
 const char* IncSet_getListItemValue(Panel* panel, int i);
 
 void IncSet_activate(IncSet* this, IncType type, Panel* panel);
 
-void IncSet_drawBar(IncSet* this);
+void IncSet_drawBar(const IncSet* this);
 
 int IncSet_synthesizeEvent(IncSet* this, int x);
 
