@@ -3,10 +3,12 @@
 /*
 htop - ListItem.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include <stdbool.h>
+
 #include "Object.h"
 
 typedef struct ListItem_ {
@@ -16,14 +18,14 @@ typedef struct ListItem_ {
    bool moving;
 } ListItem;
 
-extern ObjectClass ListItem_class;
+extern const ObjectClass ListItem_class;
 
 ListItem* ListItem_new(const char* value, int key);
 
 void ListItem_append(ListItem* this, const char* text);
 
-const char* ListItem_getRef(ListItem* this);
-
-long ListItem_compare(const void* cast1, const void* cast2);
+static inline const char* ListItem_getRef(const ListItem* this) {
+   return this->value;
+}
 
 #endif
