@@ -3,21 +3,29 @@
 /*
 htop - ProcessList.h
 (C) 2004,2005 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
-#include "Vector.h"
+#include "config.h" // IWYU pragma: keep
+
+#include <stdbool.h>
+#include <sys/types.h>
+
 #include "Hashtable.h"
-#include "UsersTable.h"
+#include "Object.h"
 #include "Panel.h"
 #include "Process.h"
+#include "RichString.h"
 #include "Settings.h"
+#include "UsersTable.h"
+#include "Vector.h"
 
 #ifdef HAVE_LIBHWLOC
 #include <hwloc.h>
 #endif
 
+
 #ifndef MAX_NAME
 #define MAX_NAME 128
 #endif
@@ -27,13 +35,16 @@ in the source distribution for its full text.
 #endif
 
 typedef struct ProcessList_ {
-   Settings* settings;
+   const Settings* settings;
 
    Vector* processes;
    Vector* processes2;
    Hashtable* processTable;
    UsersTable* usersTable;
 
+   Hashtable* displayTreeSet;
+   Hashtable* draftingTreeSet;
+
    Panel* panel;
    int following;
    uid_t userId;
@@ -52,8 +63,6 @@ typedef struct ProcessList_ {
 
    unsigned long long int totalMem;
    unsigned long long int usedMem;
-   unsigned long long int freeMem;
-   unsigned long long int sharedMem;
    unsigned long long int buffersMem;
    unsigned long long int cachedMem;
    unsigned long long int totalSwap;
@@ -62,14 +71,15 @@ typedef struct ProcessList_ {
 
    int cpuCount;
 
+   time_t scanTs;
 } ProcessList;
 
-ProcessList* ProcessList_new(UsersTable* ut, Hashtable* pidMatchList, uid_t userId);
+ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
 void ProcessList_delete(ProcessList* pl);
-void ProcessList_goThroughEntries(ProcessList* pl);
+void ProcessList_goThroughEntries(ProcessList* super, bool pauseProcessUpdate);
 
 
-ProcessList* ProcessList_init(ProcessList* this, ObjectClass* klass, UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
+ProcessList* ProcessList_init(ProcessList* this, const ObjectClass* klass, UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
 
 void ProcessList_done(ProcessList* this);
 
@@ -87,7 +97,7 @@ int ProcessList_size(ProcessList* this);
 
 void ProcessList_sort(ProcessList* this);
 
-ProcessField ProcessList_keyAt(ProcessList* this, int at);
+ProcessField ProcessList_keyAt(const ProcessList* this, int at);
 
 void ProcessList_expandTree(ProcessList* this);
 
@@ -95,6 +105,6 @@ void ProcessList_rebuildPanel(ProcessList* this);
 
 Process* ProcessList_getProcess(ProcessList* this, pid_t pid, bool* preExisting, Process_New constructor);
 
-void ProcessList_scan(ProcessList* this);
+void ProcessList_scan(ProcessList* this, bool pauseProcessUpdate);
 
 #endif
