@@ -1,26 +1,25 @@
 /*
 htop - SignalsPanel.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
-#include "Panel.h"
 #include "SignalsPanel.h"
-#include "Platform.h"
 
-#include "ListItem.h"
-#include "RichString.h"
-
-#include <stdlib.h>
-#include <assert.h>
 #include <signal.h>
+#include <stdbool.h>
 
-#include <ctype.h>
+#include "FunctionBar.h"
+#include "ListItem.h"
+#include "Object.h"
+#include "Panel.h"
+#include "Platform.h"
+#include "XUtils.h"
 
 
 Panel* SignalsPanel_new() {
-   Panel* this = Panel_new(1, 1, 1, 1, true, Class(ListItem), FunctionBar_newEnterEsc("Send   ", "Cancel "));
+   Panel* this = Panel_new(1, 1, 1, 1, Class(ListItem), true, FunctionBar_newEnterEsc("Send   ", "Cancel "));
    const int defaultSignal = SIGTERM;
    int defaultPosition = 15;
    unsigned int i;
@@ -36,7 +35,7 @@ Panel* SignalsPanel_new() {
       static char buf[16];
       for (int sig = SIGRTMIN; sig <= SIGRTMAX; i++, sig++) {
          int n = sig - SIGRTMIN;
-         xSnprintf(buf, 16, "%2d SIGRTMIN%-+3d", sig, n);
+         xSnprintf(buf, sizeof(buf), "%2d SIGRTMIN%-+3d", sig, n);
          if (n == 0) {
             buf[11] = '\0';
          }
