@@ -1,29 +1,32 @@
 /*
 htop - UptimeMeter.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "UptimeMeter.h"
-#include "Platform.h"
+
 #include "CRT.h"
+#include "Object.h"
+#include "Platform.h"
+#include "XUtils.h"
 
 
-int UptimeMeter_attributes[] = {
+static const int UptimeMeter_attributes[] = {
    UPTIME
 };
 
-static void UptimeMeter_updateValues(Meter* this, char* buffer, int len) {
+static void UptimeMeter_updateValues(Meter* this, char* buffer, size_t len) {
    int totalseconds = Platform_getUptime();
    if (totalseconds == -1) {
       xSnprintf(buffer, len, "(unknown)");
       return;
    }
    int seconds = totalseconds % 60;
-   int minutes = (totalseconds/60) % 60;
-   int hours = (totalseconds/3600) % 24;
-   int days = (totalseconds/86400);
+   int minutes = (totalseconds / 60) % 60;
+   int hours = (totalseconds / 3600) % 24;
+   int days = (totalseconds / 86400);
    this->values[0] = days;
    if (days > this->total) {
       this->total = days;
@@ -41,7 +44,7 @@ static void UptimeMeter_updateValues(Meter* this, char* buffer, int len) {
    xSnprintf(buffer, len, "%s%02d:%02d:%02d", daysbuf, hours, minutes, seconds);
 }
 
-MeterClass UptimeMeter_class = {
+const MeterClass UptimeMeter_class = {
    .super = {
       .extends = Class(Meter),
       .delete = Meter_delete
