@@ -1,28 +1,26 @@
 /*
 htop - UsersTable.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
 #include "UsersTable.h"
-#include "XAlloc.h"
 
-#include "config.h"
-
-#include <stdio.h>
-#include <string.h>
-#include <strings.h>
 #include <pwd.h>
-#include <sys/types.h>
+#include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
+
+#include "XUtils.h"
 
 
 UsersTable* UsersTable_new() {
    UsersTable* this;
    this = xMalloc(sizeof(UsersTable));
-   this->users = Hashtable_new(20, true);
+   this->users = Hashtable_new(10, true);
    return this;
 }
 
@@ -32,9 +30,9 @@ void UsersTable_delete(UsersTable* this) {
 }
 
 char* UsersTable_getRef(UsersTable* this, unsigned int uid) {
-   char* name = (char*) (Hashtable_get(this->users, uid));
+   char* name = Hashtable_get(this->users, uid);
    if (name == NULL) {
-      struct passwd* userData = getpwuid(uid);
+      const struct passwd* userData = getpwuid(uid);
       if (userData != NULL) {
          name = xStrdup(userData->pw_name);
          Hashtable_put(this->users, uid, name);
