@@ -1,32 +1,53 @@
 /*
 htop - DarwinProcess.c
 (C) 2015 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
-#include "Process.h"
 #include "DarwinProcess.h"
 
-#include <stdlib.h>
 #include <libproc.h>
-#include <string.h>
 #include <stdio.h>
-
+#include <stdlib.h>
+#include <string.h>
 #include <mach/mach.h>
 
+#include "CRT.h"
+#include "Platform.h"
+#include "Process.h"
 
-ProcessClass DarwinProcess_class = {
-   .super = {
-      .extends = Class(Process),
-      .display = Process_display,
-      .delete = Process_delete,
-      .compare = Process_compare
-   },
-   .writeField = Process_writeField,
+
+const ProcessFieldData Process_fields[LAST_PROCESSFIELD] = {
+   [0] = { .name = "", .title = NULL, .description = NULL, .flags = 0, },
+   [PID] = { .name = "PID", .title = "PID", .description = "Process/thread ID", .flags = 0, .pidColumn = true, },
+   [COMM] = { .name = "Command", .title = "Command ", .description = "Command line", .flags = 0, },
+   [STATE] = { .name = "STATE", .title = "S ", .description = "Process state (S sleeping, R running, D disk, Z zombie, T traced, W paging)", .flags = 0, },
+   [PPID] = { .name = "PPID", .title = "PPID", .description = "Parent process ID", .flags = 0, .pidColumn = true, },
+   [PGRP] = { .name = "PGRP", .title = "PGRP", .description = "Process group ID", .flags = 0, .pidColumn = true, },
+   [SESSION] = { .name = "SESSION", .title = "SID", .description = "Process's session ID", .flags = 0, .pidColumn = true, },
+   [TTY_NR] = { .name = "TTY_NR", .title = "    TTY ", .description = "Controlling terminal", .flags = 0, },
+   [TPGID] = { .name = "TPGID", .title = "TPGID", .description = "Process ID of the fg process group of the controlling terminal", .flags = 0, .pidColumn = true, },
+   [MINFLT] = { .name = "MINFLT", .title = "     MINFLT ", .description = "Number of minor faults which have not required loading a memory page from disk", .flags = 0, },
+   [MAJFLT] = { .name = "MAJFLT", .title = "     MAJFLT ", .description = "Number of major faults which have required loading a memory page from disk", .flags = 0, },
+   [PRIORITY] = { .name = "PRIORITY", .title = "PRI ", .description = "Kernel's internal priority for the process", .flags = 0, },
+   [NICE] = { .name = "NICE", .title = " NI ", .description = "Nice value (the higher the value, the more it lets other processes take priority)", .flags = 0, },
+   [STARTTIME] = { .name = "STARTTIME", .title = "START ", .description = "Time the process was started", .flags = 0, },
+
+   [PROCESSOR] = { .name = "PROCESSOR", .title = "CPU ", .description = "Id of the CPU the process last executed on", .flags = 0, },
+   [M_VIRT] = { .name = "M_VIRT", .title = " VIRT ", .description = "Total program size in virtual memory", .flags = 0, },
+   [M_RESIDENT] = { .name = "M_RESIDENT", .title = "  RES ", .description = "Resident set size, size of the text and data sections, plus stack usage", .flags = 0, },
+   [ST_UID] = { .name = "ST_UID", .title = "  UID ", .description = "User ID of the process owner", .flags = 0, },
+   [PERCENT_CPU] = { .name = "PERCENT_CPU", .title = "CPU% ", .description = "Percentage of the CPU time the process used in the last sampling", .flags = 0, },
+   [PERCENT_MEM] = { .name = "PERCENT_MEM", .title = "MEM% ", .description = "Percentage of the memory the process is using, based on resident memory size", .flags = 0, },
+   [USER] = { .name = "USER", .title = "USER      ", .description = "Username of the process owner (or user ID if name cannot be determined)", .flags = 0, },
+   [TIME] = { .name = "TIME", .title = "  TIME+  ", .description = "Total time the process has spent in user and system time", .flags = 0, },
+   [NLWP] = { .name = "NLWP", .title = "NLWP ", .description = "Number of threads in the process", .flags = 0, },
+   [TGID] = { .name = "TGID", .title = "TGID", .description = "Thread group ID (i.e. process ID)", .flags = 0, .pidColumn = true, },
+   [TRANSLATED] = { .name = "TRANSLATED", .title = "T ", .description = "Translation info (T translated, N native)", .flags = 0, },
 };
 
-DarwinProcess* DarwinProcess_new(Settings* settings) {
+Process* DarwinProcess_new(const Settings* settings) {
    DarwinProcess* this = xCalloc(1, sizeof(DarwinProcess));
    Object_setClass(this, Class(DarwinProcess));
    Process_init(&this->super, settings);
@@ -34,8 +55,9 @@ DarwinProcess* DarwinProcess_new(Settings* settings) {
    this->utime = 0;
    this->stime = 0;
    this->taskAccess = true;
+   this->translated = false;
 
-   return this;
+   return &this->super;
 }
 
 void Process_delete(Object* cast) {
@@ -45,20 +67,40 @@ void Process_delete(Object* cast) {
    free(this);
 }
 
-bool Process_isThread(Process* this) {
+static void DarwinProcess_writeField(const Process* this, RichString* str, ProcessField field) {
+   const DarwinProcess* dp = (const DarwinProcess*) this;
+   char buffer[256]; buffer[255] = '\0';
+   int attr = CRT_colors[DEFAULT_COLOR];
+   int n = sizeof(buffer) - 1;
+   switch (field) {
+   // add Platform-specific fields here
+   case TRANSLATED: xSnprintf(buffer, n, "%c ", dp->translated ? 'T' : 'N'); break;
+   default:
+      Process_writeField(this, str, field);
+      return;
+   }
+   RichString_appendWide(str, attr, buffer);
+}
+
+static int DarwinProcess_compareByKey(const Process* v1, const Process* v2, ProcessField key) {
+   const DarwinProcess* p1 = (const DarwinProcess*)v1;
+   const DarwinProcess* p2 = (const DarwinProcess*)v2;
+
+   switch (key) {
+   // add Platform-specific fields here
+   case TRANSLATED:
+      return SPACESHIP_NUMBER(p1->translated, p2->translated);
+   default:
+      return Process_compareByKey_Base(v1, v2, key);
+   }
+}
+
+bool Process_isThread(const Process* this) {
    (void) this;
    return false;
 }
 
-void DarwinProcess_setStartTime(Process *proc, struct extern_proc *ep, time_t now) {
-   struct tm date;
-
-   proc->starttime_ctime = ep->p_starttime.tv_sec;
-   (void) localtime_r(&proc->starttime_ctime, &date);
-   strftime(proc->starttime_show, 7, ((proc->starttime_ctime > now - 86400) ? "%R " : "%b%d "), &date);
-}
-
-char *DarwinProcess_getCmdLine(struct kinfo_proc* k, int* basenameOffset) {
+static char* DarwinProcess_getCmdLine(const struct kinfo_proc* k, int* basenameOffset) {
    /* This function is from the old Mac version of htop. Originally from ps? */
    int mib[3], argmax, nargs, c = 0;
    size_t size;
@@ -74,7 +116,7 @@ char *DarwinProcess_getCmdLine(struct kinfo_proc* k, int* basenameOffset) {
    }
 
    /* Allocate space for the arguments. */
-   procargs = ( char * ) xMalloc( argmax );
+   procargs = (char*)xMalloc(argmax);
    if ( procargs == NULL ) {
       goto ERROR_A;
    }
@@ -164,12 +206,12 @@ char *DarwinProcess_getCmdLine(struct kinfo_proc* k, int* basenameOffset) {
             /* Convert previous '\0'. */
             *np = ' ';
          }
-        /* Note location of current '\0'. */
-        np = cp;
-        if (*basenameOffset == 0) {
-           *basenameOffset = cp - sp;
-        }
-     }
+         /* Note location of current '\0'. */
+         np = cp;
+         if (*basenameOffset == 0) {
+            *basenameOffset = cp - sp;
+         }
+      }
    }
 
    /*
@@ -201,8 +243,10 @@ ERROR_A:
    return retval;
 }
 
-void DarwinProcess_setFromKInfoProc(Process *proc, struct kinfo_proc *ps, time_t now, bool exists) {
-   struct extern_proc *ep = &ps->kp_proc;
+void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps, bool exists) {
+   DarwinProcess* dp = (DarwinProcess*)proc;
+
+   const struct extern_proc* ep = &ps->kp_proc;
 
    /* UNSET HERE :
     *
@@ -211,14 +255,14 @@ void DarwinProcess_setFromKInfoProc(Process *proc, struct kinfo_proc *ps, time_t
     * nlwp
     * percent_cpu
     * percent_mem
-    * m_size
+    * m_virt
     * m_resident
     * minflt
     * majflt
     */
 
    /* First, the "immutable" parts */
-   if(!exists) {
+   if (!exists) {
       /* Set the PID/PGID/etc. */
       proc->pid = ep->p_pid;
       proc->ppid = ps->kp_eproc.e_ppid;
@@ -230,8 +274,11 @@ void DarwinProcess_setFromKInfoProc(Process *proc, struct kinfo_proc *ps, time_t
       /* e_tdev = (major << 24) | (minor & 0xffffff) */
       /* e_tdev == -1 for "no device" */
       proc->tty_nr = ps->kp_eproc.e_tdev & 0xff; /* TODO tty_nr is unsigned */
+      dp->translated = ps->kp_proc.p_flag & P_TRANSLATED;
+
+      proc->starttime_ctime = ep->p_starttime.tv_sec;
+      Process_fillStarttimeBuffer(proc);
 
-      DarwinProcess_setStartTime(proc, ep, now);
       proc->comm = DarwinProcess_getCmdLine(ps, &(proc->basenameOffset));
    }
 
@@ -245,29 +292,27 @@ void DarwinProcess_setFromKInfoProc(Process *proc, struct kinfo_proc *ps, time_t
    proc->updated = true;
 }
 
-void DarwinProcess_setFromLibprocPidinfo(DarwinProcess *proc, DarwinProcessList *dpl) {
+void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessList* dpl, double time_interval) {
    struct proc_taskinfo pti;
 
-   if(sizeof(pti) == proc_pidinfo(proc->super.pid, PROC_PIDTASKINFO, 0, &pti, sizeof(pti))) {
-      if(0 != proc->utime || 0 != proc->stime) {
-         uint64_t diff = (pti.pti_total_system - proc->stime)
-                  + (pti.pti_total_user - proc->utime);
+   if (sizeof(pti) == proc_pidinfo(proc->super.pid, PROC_PIDTASKINFO, 0, &pti, sizeof(pti))) {
+      uint64_t total_existing_time = proc->stime + proc->utime;
+      uint64_t total_current_time = pti.pti_total_system + pti.pti_total_user;
 
-         proc->super.percent_cpu = (double)diff * (double)dpl->super.cpuCount
-                  / ((double)dpl->global_diff * 100000.0);
-
-//       fprintf(stderr, "%f %llu %llu %llu %llu %llu\n", proc->super.percent_cpu,
-//               proc->stime, proc->utime, pti.pti_total_system, pti.pti_total_user, dpl->global_diff);
-//       exit(7);
+      if (total_existing_time && 1E-6 < time_interval) {
+         uint64_t total_time_diff = total_current_time - total_existing_time;
+         proc->super.percent_cpu = ((double)total_time_diff / time_interval) * 100.0;
+      } else {
+         proc->super.percent_cpu = 0.0;
       }
 
-      proc->super.time = (pti.pti_total_system + pti.pti_total_user) / 10000000;
+      proc->super.time = total_current_time / 10000000;
       proc->super.nlwp = pti.pti_threadnum;
-      proc->super.m_size = pti.pti_virtual_size / 1024 / PAGE_SIZE_KB;
-      proc->super.m_resident = pti.pti_resident_size / 1024 / PAGE_SIZE_KB;
+      proc->super.m_virt = pti.pti_virtual_size / ONE_K;
+      proc->super.m_resident = pti.pti_resident_size / ONE_K;
       proc->super.majflt = pti.pti_faults;
       proc->super.percent_mem = (double)pti.pti_resident_size * 100.0
-              / (double)dpl->host_info.max_mem;
+                              / (double)dpl->host_info.max_mem;
 
       proc->stime = pti.pti_total_system;
       proc->utime = pti.pti_total_user;
@@ -284,7 +329,7 @@ void DarwinProcess_setFromLibprocPidinfo(DarwinProcess *proc, DarwinProcessList
  * Based on: http://stackoverflow.com/questions/6788274/ios-mac-cpu-usage-for-thread
  * and       https://github.com/max-horvath/htop-osx/blob/e86692e869e30b0bc7264b3675d2a4014866ef46/ProcessList.c
  */
-void DarwinProcess_scanThreads(DarwinProcess *dp) {
+void DarwinProcess_scanThreads(DarwinProcess* dp) {
    Process* proc = (Process*) dp;
    kern_return_t ret;
 
@@ -346,3 +391,15 @@ void DarwinProcess_scanThreads(DarwinProcess *dp) {
    }
    proc->state = state;
 }
+
+
+const ProcessClass DarwinProcess_class = {
+   .super = {
+      .extends = Class(Process),
+      .display = Process_display,
+      .delete = Process_delete,
+      .compare = Process_compare
+   },
+   .writeField = DarwinProcess_writeField,
+   .compareByKey = DarwinProcess_compareByKey,
+};
