@@ -2,27 +2,46 @@
 htop - darwin/Platform.c
 (C) 2014 Hisham H. Muhammad
 (C) 2015 David C. Hunt
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
 #include "Platform.h"
+
+#include <errno.h>
+#include <math.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <CoreFoundation/CFString.h>
+#include <CoreFoundation/CoreFoundation.h>
+#include <IOKit/ps/IOPowerSources.h>
+#include <IOKit/ps/IOPSKeys.h>
+
+#include "ClockMeter.h"
 #include "CPUMeter.h"
+#include "CRT.h"
+#include "DarwinProcessList.h"
+#include "DateMeter.h"
+#include "DateTimeMeter.h"
+#include "HostnameMeter.h"
+#include "LoadAverageMeter.h"
+#include "Macros.h"
 #include "MemoryMeter.h"
+#include "ProcessLocksScreen.h"
 #include "SwapMeter.h"
 #include "TasksMeter.h"
-#include "LoadAverageMeter.h"
-#include "ClockMeter.h"
-#include "HostnameMeter.h"
 #include "UptimeMeter.h"
 #include "zfs/ZfsArcMeter.h"
 #include "zfs/ZfsCompressedArcMeter.h"
-#include "DarwinProcessList.h"
 
-#include <stdlib.h>
+#ifdef HAVE_MACH_MACH_TIME_H
+#include <mach/mach_time.h>
+#endif
 
 
-ProcessField Platform_defaultFields[] = { PID, USER, PRIORITY, NICE, M_SIZE, M_RESIDENT, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, 0 };
+const ProcessField Platform_defaultFields[] = { PID, USER, PRIORITY, NICE, M_VIRT, M_RESIDENT, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, 0 };
 
 const SignalItem Platform_signals[] = {
    { .name = " 0 Cancel",    .number =  0 },
@@ -60,40 +79,13 @@ const SignalItem Platform_signals[] = {
    { .name = "31 SIGUSR2",   .number = 31 },
 };
 
-const unsigned int Platform_numberOfSignals = sizeof(Platform_signals)/sizeof(SignalItem);
+const unsigned int Platform_numberOfSignals = ARRAYSIZE(Platform_signals);
 
-ProcessFieldData Process_fields[] = {
-   [0] = { .name = "", .title = NULL, .description = NULL, .flags = 0, },
-   [PID] = { .name = "PID", .title = "    PID ", .description = "Process/thread ID", .flags = 0, },
-   [COMM] = { .name = "Command", .title = "Command ", .description = "Command line", .flags = 0, },
-   [STATE] = { .name = "STATE", .title = "S ", .description = "Process state (S sleeping, R running, D disk, Z zombie, T traced, W paging)", .flags = 0, },
-   [PPID] = { .name = "PPID", .title = "   PPID ", .description = "Parent process ID", .flags = 0, },
-   [PGRP] = { .name = "PGRP", .title = "   PGRP ", .description = "Process group ID", .flags = 0, },
-   [SESSION] = { .name = "SESSION", .title = "    SID ", .description = "Process's session ID", .flags = 0, },
-   [TTY_NR] = { .name = "TTY_NR", .title = "    TTY ", .description = "Controlling terminal", .flags = 0, },
-   [TPGID] = { .name = "TPGID", .title = "  TPGID ", .description = "Process ID of the fg process group of the controlling terminal", .flags = 0, },
-   [MINFLT] = { .name = "MINFLT", .title = "     MINFLT ", .description = "Number of minor faults which have not required loading a memory page from disk", .flags = 0, },
-   [MAJFLT] = { .name = "MAJFLT", .title = "     MAJFLT ", .description = "Number of major faults which have required loading a memory page from disk", .flags = 0, },
-   [PRIORITY] = { .name = "PRIORITY", .title = "PRI ", .description = "Kernel's internal priority for the process", .flags = 0, },
-   [NICE] = { .name = "NICE", .title = " NI ", .description = "Nice value (the higher the value, the more it lets other processes take priority)", .flags = 0, },
-   [STARTTIME] = { .name = "STARTTIME", .title = "START ", .description = "Time the process was started", .flags = 0, },
-
-   [PROCESSOR] = { .name = "PROCESSOR", .title = "CPU ", .description = "Id of the CPU the process last executed on", .flags = 0, },
-   [M_SIZE] = { .name = "M_SIZE", .title = " VIRT ", .description = "Total program size in virtual memory", .flags = 0, },
-   [M_RESIDENT] = { .name = "M_RESIDENT", .title = "  RES ", .description = "Resident set size, size of the text and data sections, plus stack usage", .flags = 0, },
-   [ST_UID] = { .name = "ST_UID", .title = "  UID ", .description = "User ID of the process owner", .flags = 0, },
-   [PERCENT_CPU] = { .name = "PERCENT_CPU", .title = "CPU% ", .description = "Percentage of the CPU time the process used in the last sampling", .flags = 0, },
-   [PERCENT_MEM] = { .name = "PERCENT_MEM", .title = "MEM% ", .description = "Percentage of the memory the process is using, based on resident memory size", .flags = 0, },
-   [USER] = { .name = "USER", .title = "USER      ", .description = "Username of the process owner (or user ID if name cannot be determined)", .flags = 0, },
-   [TIME] = { .name = "TIME", .title = "  TIME+  ", .description = "Total time the process has spent in user and system time", .flags = 0, },
-   [NLWP] = { .name = "NLWP", .title = "NLWP ", .description = "Number of threads in the process", .flags = 0, },
-   [TGID] = { .name = "TGID", .title = "   TGID ", .description = "Thread group ID (i.e. process ID)", .flags = 0, },
-   [100] = { .name = "*** report bug! ***", .title = NULL, .description = NULL, .flags = 0, },
-};
-
-MeterClass* Platform_meterTypes[] = {
+const MeterClass* const Platform_meterTypes[] = {
    &CPUMeter_class,
    &ClockMeter_class,
+   &DateMeter_class,
+   &DateTimeMeter_class,
    &LoadAverageMeter_class,
    &LoadMeter_class,
    &MemoryMeter_class,
@@ -104,21 +96,54 @@ MeterClass* Platform_meterTypes[] = {
    &UptimeMeter_class,
    &AllCPUsMeter_class,
    &AllCPUs2Meter_class,
+   &AllCPUs4Meter_class,
+   &AllCPUs8Meter_class,
    &LeftCPUsMeter_class,
    &RightCPUsMeter_class,
    &LeftCPUs2Meter_class,
    &RightCPUs2Meter_class,
+   &LeftCPUs4Meter_class,
+   &RightCPUs4Meter_class,
+   &LeftCPUs8Meter_class,
+   &RightCPUs8Meter_class,
    &ZfsArcMeter_class,
    &ZfsCompressedArcMeter_class,
    &BlankMeter_class,
    NULL
 };
 
-void Platform_setBindings(Htop_Action* keys) {
-   (void) keys;
+double Platform_timebaseToNS = 1.0;
+
+long Platform_clockTicksPerSec = -1;
+
+void Platform_init(void) {
+   // Check if we can determine the timebase used on this system.
+   // If the API is unavailable assume we get our timebase in nanoseconds.
+#ifdef HAVE_MACH_TIMEBASE_INFO
+   mach_timebase_info_data_t info;
+   mach_timebase_info(&info);
+   Platform_timebaseToNS = (double)info.numer / (double)info.denom;
+#else
+   Platform_timebaseToNS = 1.0;
+#endif
+
+   // Determine the number of clock ticks per second
+   errno = 0;
+   Platform_clockTicksPerSec = sysconf(_SC_CLK_TCK);
+
+   if (errno || Platform_clockTicksPerSec < 1) {
+      CRT_fatalError("Unable to retrieve clock tick rate");
+   }
 }
 
-int Platform_numberOfFields = 100;
+void Platform_done(void) {
+   /* no platform-specific cleanup needed */
+}
+
+void Platform_setBindings(Htop_Action* keys) {
+   /* no platform-specific key bindings */
+   (void) keys;
+}
 
 int Platform_getUptime() {
    struct timeval bootTime, currTime;
@@ -137,7 +162,7 @@ int Platform_getUptime() {
 void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
    double results[3];
 
-   if(3 == getloadavg(results, 3)) {
+   if (3 == getloadavg(results, 3)) {
       *one = results[0];
       *five = results[1];
       *fifteen = results[2];
@@ -153,19 +178,9 @@ int Platform_getMaxPid() {
    return 99999;
 }
 
-ProcessPidColumn Process_pidColumns[] = {
-   { .id = PID, .label = "PID" },
-   { .id = PPID, .label = "PPID" },
-   { .id = TPGID, .label = "TPGID" },
-   { .id = TGID, .label = "TGID" },
-   { .id = PGRP, .label = "PGRP" },
-   { .id = SESSION, .label = "SID" },
-   { .id = 0, .label = NULL },
-};
-
 static double Platform_setCPUAverageValues(Meter* mtr) {
-   DarwinProcessList *dpl = (DarwinProcessList *)mtr->pl;
-   int cpus = dpl->super.cpuCount;
+   const ProcessList* dpl = mtr->pl;
+   int cpus = dpl->cpuCount;
    double sumNice = 0.0;
    double sumNormal = 0.0;
    double sumKernel = 0.0;
@@ -188,36 +203,37 @@ double Platform_setCPUValues(Meter* mtr, int cpu) {
       return Platform_setCPUAverageValues(mtr);
    }
 
-   DarwinProcessList *dpl = (DarwinProcessList *)mtr->pl;
-   processor_cpu_load_info_t prev = &dpl->prev_load[cpu-1];
-   processor_cpu_load_info_t curr = &dpl->curr_load[cpu-1];
+   const DarwinProcessList* dpl = (const DarwinProcessList*)mtr->pl;
+   const processor_cpu_load_info_t prev = &dpl->prev_load[cpu - 1];
+   const processor_cpu_load_info_t curr = &dpl->curr_load[cpu - 1];
    double total = 0;
 
    /* Take the sums */
-   for(size_t i = 0; i < CPU_STATE_MAX; ++i) {
+   for (size_t i = 0; i < CPU_STATE_MAX; ++i) {
       total += (double)curr->cpu_ticks[i] - (double)prev->cpu_ticks[i];
    }
 
    mtr->values[CPU_METER_NICE]
-           = ((double)curr->cpu_ticks[CPU_STATE_NICE] - (double)prev->cpu_ticks[CPU_STATE_NICE])* 100.0 / total;
+      = ((double)curr->cpu_ticks[CPU_STATE_NICE] - (double)prev->cpu_ticks[CPU_STATE_NICE]) * 100.0 / total;
    mtr->values[CPU_METER_NORMAL]
-           = ((double)curr->cpu_ticks[CPU_STATE_USER] - (double)prev->cpu_ticks[CPU_STATE_USER])* 100.0 / total;
+      = ((double)curr->cpu_ticks[CPU_STATE_USER] - (double)prev->cpu_ticks[CPU_STATE_USER]) * 100.0 / total;
    mtr->values[CPU_METER_KERNEL]
-           = ((double)curr->cpu_ticks[CPU_STATE_SYSTEM] - (double)prev->cpu_ticks[CPU_STATE_SYSTEM])* 100.0 / total;
+      = ((double)curr->cpu_ticks[CPU_STATE_SYSTEM] - (double)prev->cpu_ticks[CPU_STATE_SYSTEM]) * 100.0 / total;
 
-   Meter_setItems(mtr, 3);
+   mtr->curItems = 3;
 
    /* Convert to percent and return */
    total = mtr->values[CPU_METER_NICE] + mtr->values[CPU_METER_NORMAL] + mtr->values[CPU_METER_KERNEL];
 
-   mtr->values[CPU_METER_FREQUENCY] = -1;
+   mtr->values[CPU_METER_FREQUENCY] = NAN;
+   mtr->values[CPU_METER_TEMPERATURE] = NAN;
 
    return CLAMP(total, 0.0, 100.0);
 }
 
 void Platform_setMemoryValues(Meter* mtr) {
-   DarwinProcessList *dpl = (DarwinProcessList *)mtr->pl;
-   vm_statistics_t vm = &dpl->vm_stats;
+   const DarwinProcessList* dpl = (const DarwinProcessList*)mtr->pl;
+   const struct vm_statistics* vm = &dpl->vm_stats;
    double page_K = (double)vm_page_size / (double)1024;
 
    mtr->total = dpl->host_info.max_mem / 1024;
@@ -227,23 +243,23 @@ void Platform_setMemoryValues(Meter* mtr) {
 }
 
 void Platform_setSwapValues(Meter* mtr) {
-  int mib[2] = {CTL_VM, VM_SWAPUSAGE};
-  struct xsw_usage swapused;
-  size_t swlen = sizeof(swapused);
-  sysctl(mib, 2, &swapused, &swlen, NULL, 0);
+   int mib[2] = {CTL_VM, VM_SWAPUSAGE};
+   struct xsw_usage swapused;
+   size_t swlen = sizeof(swapused);
+   sysctl(mib, 2, &swapused, &swlen, NULL, 0);
 
-  mtr->total = swapused.xsu_total / 1024;
-  mtr->values[0] = swapused.xsu_used / 1024;
+   mtr->total = swapused.xsu_total / 1024;
+   mtr->values[0] = swapused.xsu_used / 1024;
 }
 
 void Platform_setZfsArcValues(Meter* this) {
-   DarwinProcessList* dpl = (DarwinProcessList*) this->pl;
+   const DarwinProcessList* dpl = (const DarwinProcessList*) this->pl;
 
    ZfsArcMeter_readStats(this, &(dpl->zfs));
 }
 
 void Platform_setZfsCompressedArcValues(Meter* this) {
-   DarwinProcessList* dpl = (DarwinProcessList*) this->pl;
+   const DarwinProcessList* dpl = (const DarwinProcessList*) this->pl;
 
    ZfsCompressedArcMeter_readStats(this, &(dpl->zfs));
 }
@@ -263,33 +279,33 @@ char* Platform_getProcessEnv(pid_t pid) {
          mib[0] = CTL_KERN;
          mib[1] = KERN_PROCARGS2;
          mib[2] = pid;
-         size_t bufsz = argmax;
+         bufsz = argmax;
          if (sysctl(mib, 3, buf, &bufsz, 0, 0) == 0) {
             if (bufsz > sizeof(int)) {
                char *p = buf, *endp = buf + bufsz;
-               int argc = *(int*)p;
+               int argc = *(int*)(void*)p;
                p += sizeof(int);
 
                // skip exe
-               p = strchr(p, 0)+1;
+               p = strchr(p, 0) + 1;
 
                // skip padding
-               while(!*p && p < endp)
+               while (!*p && p < endp)
                   ++p;
 
                // skip argv
-               for (; argc-- && p < endp; p = strrchr(p, 0)+1)
+               for (; argc-- && p < endp; p = strrchr(p, 0) + 1)
                   ;
 
                // skip padding
-               while(!*p && p < endp)
+               while (!*p && p < endp)
                   ++p;
 
                size_t size = endp - p;
-               env = xMalloc(size+2);
+               env = xMalloc(size + 2);
                memcpy(env, p, size);
                env[size] = 0;
-               env[size+1] = 0;
+               env[size + 1] = 0;
             }
          }
          free(buf);
@@ -298,3 +314,96 @@ char* Platform_getProcessEnv(pid_t pid) {
 
    return env;
 }
+
+char* Platform_getInodeFilename(pid_t pid, ino_t inode) {
+    (void)pid;
+    (void)inode;
+    return NULL;
+}
+
+FileLocks_ProcessData* Platform_getProcessLocks(pid_t pid) {
+    (void)pid;
+    return NULL;
+}
+
+bool Platform_getDiskIO(DiskIOData* data) {
+   // TODO
+   (void)data;
+   return false;
+}
+
+bool Platform_getNetworkIO(unsigned long int* bytesReceived,
+                           unsigned long int* packetsReceived,
+                           unsigned long int* bytesTransmitted,
+                           unsigned long int* packetsTransmitted) {
+   // TODO
+   *bytesReceived = 0;
+   *packetsReceived = 0;
+   *bytesTransmitted = 0;
+   *packetsTransmitted = 0;
+   return false;
+}
+
+void Platform_getBattery(double* percent, ACPresence* isOnAC) {
+   CFTypeRef power_sources = IOPSCopyPowerSourcesInfo();
+
+   *percent = NAN;
+   *isOnAC = AC_ERROR;
+
+   if (NULL == power_sources)
+      return;
+
+   CFArrayRef list = IOPSCopyPowerSourcesList(power_sources);
+   CFDictionaryRef battery = NULL;
+   int len;
+
+   if (NULL == list) {
+      CFRelease(power_sources);
+
+      return;
+   }
+
+   len = CFArrayGetCount(list);
+
+   /* Get the battery */
+   for (int i = 0; i < len && battery == NULL; ++i) {
+      CFDictionaryRef candidate = IOPSGetPowerSourceDescription(power_sources,
+                                  CFArrayGetValueAtIndex(list, i)); /* GET rule */
+      CFStringRef type;
+
+      if (NULL != candidate) {
+         type = (CFStringRef) CFDictionaryGetValue(candidate,
+                CFSTR(kIOPSTransportTypeKey)); /* GET rule */
+
+         if (kCFCompareEqualTo == CFStringCompare(type, CFSTR(kIOPSInternalType), 0)) {
+            CFRetain(candidate);
+            battery = candidate;
+         }
+      }
+   }
+
+   if (NULL != battery) {
+      /* Determine the AC state */
+      CFStringRef power_state = CFDictionaryGetValue(battery, CFSTR(kIOPSPowerSourceStateKey));
+
+      *isOnAC = (kCFCompareEqualTo == CFStringCompare(power_state, CFSTR(kIOPSACPowerValue), 0))
+              ? AC_PRESENT
+              : AC_ABSENT;
+
+      /* Get the percentage remaining */
+      double current;
+      double max;
+
+      CFNumberGetValue(CFDictionaryGetValue(battery, CFSTR(kIOPSCurrentCapacityKey)),
+                       kCFNumberDoubleType, &current);
+      CFNumberGetValue(CFDictionaryGetValue(battery, CFSTR(kIOPSMaxCapacityKey)),
+                       kCFNumberDoubleType, &max);
+
+      *percent = (current * 100.0) / max;
+
+      CFRelease(battery);
+   }
+
+   CFRelease(list);
+   CFRelease(power_sources);
+}
