@@ -18,16 +18,16 @@ in the source distribution for its full text.
 
 const char* const nodevStr = "nodev";
 
-ProcessFieldData Process_fields[] = {
+const ProcessFieldData Process_fields[LAST_PROCESSFIELD] = {
    [0] = { .name = "", .title = NULL, .description = NULL, .flags = 0, },
-   [PID] = { .name = "PID", .title = "    PID ", .description = "Process/thread ID", .flags = 0, },
+   [PID] = { .name = "PID", .title = "PID", .description = "Process/thread ID", .flags = 0, .pidColumn = true, },
    [COMM] = { .name = "Command", .title = "Command ", .description = "Command line", .flags = 0, },
    [STATE] = { .name = "STATE", .title = "S ", .description = "Process state (S sleeping, R running, D disk, Z zombie, T traced, W paging)", .flags = 0, },
-   [PPID] = { .name = "PPID", .title = "   PPID ", .description = "Parent process ID", .flags = 0, },
-   [PGRP] = { .name = "PGRP", .title = "   PGRP ", .description = "Process group ID", .flags = 0, },
-   [SESSION] = { .name = "SESSION", .title = "    SID ", .description = "Process's session ID", .flags = 0, },
+   [PPID] = { .name = "PPID", .title = "PPID", .description = "Parent process ID", .flags = 0, .pidColumn = true, },
+   [PGRP] = { .name = "PGRP", .title = "PGRP", .description = "Process group ID", .flags = 0, .pidColumn = true, },
+   [SESSION] = { .name = "SESSION", .title = "SID", .description = "Process's session ID", .flags = 0, .pidColumn = true, },
    [TTY_NR] = { .name = "TTY_NR", .title = "    TTY ", .description = "Controlling terminal", .flags = PROCESS_FLAG_FREEBSD_TTY, },
-   [TPGID] = { .name = "TPGID", .title = "  TPGID ", .description = "Process ID of the fg process group of the controlling terminal", .flags = 0, },
+   [TPGID] = { .name = "TPGID", .title = "TPGID", .description = "Process ID of the fg process group of the controlling terminal", .flags = 0, .pidColumn = true, },
    [MINFLT] = { .name = "MINFLT", .title = "     MINFLT ", .description = "Number of minor faults which have not required loading a memory page from disk", .flags = 0, },
    [MAJFLT] = { .name = "MAJFLT", .title = "     MAJFLT ", .description = "Number of major faults which have required loading a memory page from disk", .flags = 0, },
    [PRIORITY] = { .name = "PRIORITY", .title = "PRI ", .description = "Kernel's internal priority for the process", .flags = 0, },
@@ -44,21 +44,9 @@ ProcessFieldData Process_fields[] = {
    [USER] = { .name = "USER", .title = "USER      ", .description = "Username of the process owner (or user ID if name cannot be determined)", .flags = 0, },
    [TIME] = { .name = "TIME", .title = "  TIME+  ", .description = "Total time the process has spent in user and system time", .flags = 0, },
    [NLWP] = { .name = "NLWP", .title = "NLWP ", .description = "Number of threads in the process", .flags = 0, },
-   [TGID] = { .name = "TGID", .title = "   TGID ", .description = "Thread group ID (i.e. process ID)", .flags = 0, },
-   [JID] = { .name = "JID", .title = "    JID ", .description = "Jail prison ID", .flags = 0, },
+   [TGID] = { .name = "TGID", .title = "TGID", .description = "Thread group ID (i.e. process ID)", .flags = 0, .pidColumn = true, },
+   [JID] = { .name = "JID", .title = "JID", .description = "Jail prison ID", .flags = 0, .pidColumn = true, },
    [JAIL] = { .name = "JAIL", .title = "JAIL        ", .description = "Jail prison name", .flags = 0, },
-   [LAST_PROCESSFIELD] = { .name = "*** report bug! ***", .title = NULL, .description = NULL, .flags = 0, },
-};
-
-ProcessPidColumn Process_pidColumns[] = {
-   { .id = JID, .label = "JID" },
-   { .id = PID, .label = "PID" },
-   { .id = PPID, .label = "PPID" },
-   { .id = TPGID, .label = "TPGID" },
-   { .id = TGID, .label = "TGID" },
-   { .id = PGRP, .label = "PGRP" },
-   { .id = SESSION, .label = "SID" },
-   { .id = 0, .label = NULL },
 };
 
 Process* FreeBSDProcess_new(const Settings* settings) {
@@ -80,17 +68,12 @@ static void FreeBSDProcess_writeField(const Process* this, RichString* str, Proc
    char buffer[256]; buffer[255] = '\0';
    int attr = CRT_colors[DEFAULT_COLOR];
    int n = sizeof(buffer) - 1;
-   switch ((int) field) {
+   switch (field) {
    // add FreeBSD-specific fields here
-   case JID: xSnprintf(buffer, n, Process_pidFormat, fp->jid); break;
-   case JAIL: {
-      xSnprintf(buffer, n, "%-11s ", fp->jname);
-      if (buffer[11] != '\0') {
-         buffer[11] = ' ';
-         buffer[12] = '\0';
-      }
-      break;
-   }
+   case JID: xSnprintf(buffer, n, "%*d ", Process_pidDigits, fp->jid); break;
+   case JAIL:
+      Process_printLeftAlignedField(str, attr, fp->jname ? fp->jname : "N/A", 11);
+      return;
    case TTY_NR:
       if (fp->ttyPath) {
          if (fp->ttyPath == nodevStr)
@@ -105,22 +88,14 @@ static void FreeBSDProcess_writeField(const Process* this, RichString* str, Proc
       Process_writeField(this, str, field);
       return;
    }
-   RichString_append(str, attr, buffer);
+   RichString_appendWide(str, attr, buffer);
 }
 
-static long FreeBSDProcess_compare(const void* v1, const void* v2) {
-   const FreeBSDProcess *p1, *p2;
-   const Settings *settings = ((const Process*)v1)->settings;
+static int FreeBSDProcess_compareByKey(const Process* v1, const Process* v2, ProcessField key) {
+   const FreeBSDProcess* p1 = (const FreeBSDProcess*)v1;
+   const FreeBSDProcess* p2 = (const FreeBSDProcess*)v2;
 
-   if (settings->direction == 1) {
-      p1 = (const FreeBSDProcess*)v1;
-      p2 = (const FreeBSDProcess*)v2;
-   } else {
-      p2 = (const FreeBSDProcess*)v1;
-      p1 = (const FreeBSDProcess*)v2;
-   }
-
-   switch ((int) settings->sortKey) {
+   switch (key) {
    // add FreeBSD-specific fields here
    case JID:
       return SPACESHIP_NUMBER(p1->jid, p2->jid);
@@ -129,7 +104,7 @@ static long FreeBSDProcess_compare(const void* v1, const void* v2) {
    case TTY_NR:
       return SPACESHIP_NULLSTR(p1->ttyPath, p2->ttyPath);
    default:
-      return Process_compare(v1, v2);
+      return Process_compareByKey_Base(v1, v2, key);
    }
 }
 
@@ -148,7 +123,8 @@ const ProcessClass FreeBSDProcess_class = {
       .extends = Class(Process),
       .display = Process_display,
       .delete = Process_delete,
-      .compare = FreeBSDProcess_compare
+      .compare = Process_compare
    },
    .writeField = FreeBSDProcess_writeField,
+   .compareByKey = FreeBSDProcess_compareByKey
 };
