@@ -3,7 +3,7 @@
 /*
 htop - IOPriority.h
 (C) 2004-2012 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 
 Based on ionice,
@@ -28,7 +28,7 @@ enum {
 
 typedef int IOPriority;
 
-#define IOPriority_tuple(class_, data_) (((class_) << IOPRIO_CLASS_SHIFT) | data_)
+#define IOPriority_tuple(class_, data_) (((class_) << IOPRIO_CLASS_SHIFT) | (data_))
 
 #define IOPriority_error 0xffffffff
 
