@@ -3,11 +3,12 @@ htop - solaris/Platform.c
 (C) 2014 Hisham H. Muhammad
 (C) 2015 David C. Hunt
 (C) 2017,2018 Guy M. Broome
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "Platform.h"
+#include "Macros.h"
 #include "Meter.h"
 #include "CPUMeter.h"
 #include "MemoryMeter.h"
@@ -15,6 +16,8 @@ in the source distribution for its full text.
 #include "TasksMeter.h"
 #include "LoadAverageMeter.h"
 #include "ClockMeter.h"
+#include "DateMeter.h"
+#include "DateTimeMeter.h"
 #include "HostnameMeter.h"
 #include "UptimeMeter.h"
 #include "zfs/ZfsArcMeter.h"
@@ -81,13 +84,15 @@ const SignalItem Platform_signals[] = {
    { .name = "41 SIGINFO",     .number = 41 },
 };
 
-const unsigned int Platform_numberOfSignals = sizeof(Platform_signals)/sizeof(SignalItem);
+const unsigned int Platform_numberOfSignals = ARRAYSIZE(Platform_signals);
 
-ProcessField Platform_defaultFields[] = { PID, LWPID, USER, PRIORITY, NICE, M_SIZE, M_RESIDENT, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, 0 };
+const ProcessField Platform_defaultFields[] = { PID, LWPID, USER, PRIORITY, NICE, M_VIRT, M_RESIDENT, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, 0 };
 
-MeterClass* Platform_meterTypes[] = {
+const MeterClass* const Platform_meterTypes[] = {
    &CPUMeter_class,
    &ClockMeter_class,
+   &DateMeter_class,
+   &DateTimeMeter_class,
    &LoadAverageMeter_class,
    &LoadMeter_class,
    &MemoryMeter_class,
@@ -98,28 +103,39 @@ MeterClass* Platform_meterTypes[] = {
    &UptimeMeter_class,
    &AllCPUsMeter_class,
    &AllCPUs2Meter_class,
+   &AllCPUs4Meter_class,
+   &AllCPUs8Meter_class,
    &LeftCPUsMeter_class,
    &RightCPUsMeter_class,
    &LeftCPUs2Meter_class,
    &RightCPUs2Meter_class,
+   &LeftCPUs4Meter_class,
+   &RightCPUs4Meter_class,
+   &LeftCPUs8Meter_class,
+   &RightCPUs8Meter_class,
    &ZfsArcMeter_class,
    &ZfsCompressedArcMeter_class,
    &BlankMeter_class,
    NULL
 };
 
-void Platform_setBindings(Htop_Action* keys) {
-   (void) keys;
+void Platform_init(void) {
+   /* no platform-specific setup needed */
 }
 
-int Platform_numberOfFields = LAST_PROCESSFIELD;
+void Platform_done(void) {
+   /* no platform-specific cleanup needed */
+}
 
-extern char Process_pidFormat[20];
+void Platform_setBindings(Htop_Action* keys) {
+   /* no platform-specific key bindings */
+   (void) keys;
+}
 
 int Platform_getUptime() {
    int boot_time = 0;
    int curr_time = time(NULL);
-   struct utmpx * ent;
+   struct utmpx* ent;
 
    while (( ent = getutxent() )) {
       if ( !strcmp("system boot", ent->ut_line )) {
@@ -129,7 +145,7 @@ int Platform_getUptime() {
 
    endutxent();
 
-   return (curr_time-boot_time);
+   return (curr_time - boot_time);
 }
 
 void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
@@ -140,31 +156,35 @@ void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
 }
 
 int Platform_getMaxPid() {
-   kstat_ctl_t *kc = NULL;
-   kstat_t *kshandle = NULL;
-   kvar_t *ksvar = NULL;
    int vproc = 32778; // Reasonable Solaris default
-   kc = kstat_open();
-   if (kc != NULL) { kshandle = kstat_lookup(kc,"unix",0,"var"); }
-   if (kshandle != NULL) { kstat_read(kc,kshandle,NULL); }
-   ksvar = kshandle->ks_data;
-   if (ksvar->v_proc > 0 ) {
-      vproc = ksvar->v_proc;
+
+   kstat_ctl_t* kc = kstat_open();
+   if (kc != NULL) {
+      kstat_t* kshandle = kstat_lookup(kc, "unix", 0, "var");
+      if (kshandle != NULL) {
+         kstat_read(kc, kshandle, NULL);
+
+         kvar_t* ksvar = kshandle->ks_data;
+         if (ksvar && ksvar->v_proc > 0) {
+            vproc = ksvar->v_proc;
+         }
+      }
+      kstat_close(kc);
    }
-   if (kc != NULL) { kstat_close(kc); }
+
    return vproc;
 }
 
 double Platform_setCPUValues(Meter* this, int cpu) {
-   SolarisProcessList* spl = (SolarisProcessList*) this->pl;
+   const SolarisProcessList* spl = (const SolarisProcessList*) this->pl;
    int cpus = this->pl->cpuCount;
-   CPUData* cpuData = NULL;
+   const CPUData* cpuData = NULL;
 
    if (cpus == 1) {
-     // single CPU box has everything in spl->cpus[0]
-     cpuData = &(spl->cpus[0]);
+      // single CPU box has everything in spl->cpus[0]
+      cpuData = &(spl->cpus[0]);
    } else {
-     cpuData = &(spl->cpus[cpu]);
+      cpuData = &(spl->cpus[cpu]);
    }
 
    double percent;
@@ -175,24 +195,24 @@ double Platform_setCPUValues(Meter* this, int cpu) {
    if (this->pl->settings->detailedCPUTime) {
       v[CPU_METER_KERNEL]  = cpuData->systemPercent;
       v[CPU_METER_IRQ]     = cpuData->irqPercent;
-      Meter_setItems(this, 4);
-      percent = v[0]+v[1]+v[2]+v[3];
+      this->curItems = 4;
+      percent = v[0] + v[1] + v[2] + v[3];
    } else {
       v[2] = cpuData->systemAllPercent;
-      Meter_setItems(this, 3);
-      percent = v[0]+v[1]+v[2];
+      this->curItems = 3;
+      percent = v[0] + v[1] + v[2];
    }
 
-   percent = CLAMP(percent, 0.0, 100.0);
-   if (isnan(percent)) percent = 0.0;
+   percent = isnan(percent) ? 0.0 : CLAMP(percent, 0.0, 100.0);
 
-   v[CPU_METER_FREQUENCY] = -1;
+   v[CPU_METER_FREQUENCY] = cpuData->frequency;
+   v[CPU_METER_TEMPERATURE] = NAN;
 
    return percent;
 }
 
 void Platform_setMemoryValues(Meter* this) {
-   ProcessList* pl = (ProcessList*) this->pl;
+   const ProcessList* pl = this->pl;
    this->total = pl->totalMem;
    this->values[0] = pl->usedMem;
    this->values[1] = pl->buffersMem;
@@ -200,32 +220,34 @@ void Platform_setMemoryValues(Meter* this) {
 }
 
 void Platform_setSwapValues(Meter* this) {
-   ProcessList* pl = (ProcessList*) this->pl;
+   const ProcessList* pl = this->pl;
    this->total = pl->totalSwap;
    this->values[0] = pl->usedSwap;
 }
 
 void Platform_setZfsArcValues(Meter* this) {
-   SolarisProcessList* spl = (SolarisProcessList*) this->pl;
+   const SolarisProcessList* spl = (const SolarisProcessList*) this->pl;
 
    ZfsArcMeter_readStats(this, &(spl->zfs));
 }
 
 void Platform_setZfsCompressedArcValues(Meter* this) {
-   SolarisProcessList* spl = (SolarisProcessList*) this->pl;
+   const SolarisProcessList* spl = (const SolarisProcessList*) this->pl;
 
    ZfsCompressedArcMeter_readStats(this, &(spl->zfs));
 }
 
-static int Platform_buildenv(void *accum, struct ps_prochandle *Phandle, uintptr_t addr, const char *str) {
-   envAccum *accump = accum;
+static int Platform_buildenv(void* accum, struct ps_prochandle* Phandle, uintptr_t addr, const char* str) {
+   envAccum* accump = accum;
    (void) Phandle;
    (void) addr;
    size_t thissz = strlen(str);
-   if ((thissz + 2) > (accump->capacity - accump->size))
+   if ((thissz + 2) > (accump->capacity - accump->size)) {
       accump->env = xRealloc(accump->env, accump->capacity *= 2);
-   if ((thissz + 2) > (accump->capacity - accump->size))
+   }
+   if ((thissz + 2) > (accump->capacity - accump->size)) {
       return 1;
+   }
    strlcpy( accump->env + accump->size, str, (accump->capacity - accump->size));
    strncpy( accump->env + accump->size + thissz + 1, "\n", 1);
    accump->size = accump->size + thissz + 1;
@@ -236,19 +258,54 @@ char* Platform_getProcessEnv(pid_t pid) {
    envAccum envBuilder;
    pid_t realpid = pid / 1024;
    int graberr;
-   struct ps_prochandle *Phandle;
+   struct ps_prochandle* Phandle;
 
-   if ((Phandle = Pgrab(realpid,PGRAB_RDONLY,&graberr)) == NULL)
+   if ((Phandle = Pgrab(realpid, PGRAB_RDONLY, &graberr)) == NULL) {
       return "Unable to read process environment.";
+   }
 
    envBuilder.capacity = 4096;
    envBuilder.size     = 0;
    envBuilder.env      = xMalloc(envBuilder.capacity);
 
-   (void) Penv_iter(Phandle,Platform_buildenv,&envBuilder);
+   (void) Penv_iter(Phandle, Platform_buildenv, &envBuilder);
 
    Prelease(Phandle, 0);
 
    strncpy( envBuilder.env + envBuilder.size, "\0", 1);
    return envBuilder.env;
 }
+
+char* Platform_getInodeFilename(pid_t pid, ino_t inode) {
+    (void)pid;
+    (void)inode;
+    return NULL;
+}
+
+FileLocks_ProcessData* Platform_getProcessLocks(pid_t pid) {
+    (void)pid;
+    return NULL;
+}
+
+bool Platform_getDiskIO(DiskIOData* data) {
+   // TODO
+   (void)data;
+   return false;
+}
+
+bool Platform_getNetworkIO(unsigned long int* bytesReceived,
+                           unsigned long int* packetsReceived,
+                           unsigned long int* bytesTransmitted,
+                           unsigned long int* packetsTransmitted) {
+   // TODO
+   *bytesReceived = 0;
+   *packetsReceived = 0;
+   *bytesTransmitted = 0;
+   *packetsTransmitted = 0;
+   return false;
+}
+
+void Platform_getBattery(double* percent, ACPresence* isOnAC) {
+   *percent = NAN;
+   *isOnAC = AC_ERROR;
+}
